package com.gydx.component;

import java.io.IOException;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArraySet;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.gydx.service.BiddingMesService;
import com.gydx.service.RuleManagementService;
import com.gydx.vo.BiddingMesVO;
import com.gydx.vo.FoodRuleVO;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 *  --------------------- 竞价信息 ---------------------
 *  1.进入页面直接建立socket链接
 *  2.查找所有竞价单号，对应状态为进行中的，依据点击时间判断倒计时
 *  3.将倒计时变更的信息更新到页面，直到链接关闭或者倒计时变为
 */
@ServerEndpoint(value = "/client/BiddingMes/{BiddingMesId}")
@Component
public class WSBiddingMes {
    private static Logger logger = LogManager.getLogger(WSBiddingMes.class.getName());

    /**
     * concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
     */
    private static CopyOnWriteArraySet<WSBiddingMes> wsClientMap = new CopyOnWriteArraySet<>();

    private static BiddingMesService biddingMesService;

    @Autowired
    public void setSocketMessageHandler(BiddingMesService messageHandler) {
        biddingMesService = messageHandler;
    }

    private static RuleManagementService ruleManagementService;

    @Autowired
    public void setSocketMessageRuleID(RuleManagementService messageHandler) {
        ruleManagementService = messageHandler;
    }


    /**
     * 与某个客户端的连接会话，需要通过它来给客户端发送数据
     */
    private Session session;

    /**
     * 连接建立成功调用的方法
     *
     * @param session 当前会话session
     */
    @OnOpen
    public void onOpen(@PathParam("BiddingMesId") String BiddingMesId, Session session) {
        this.session = session;
        wsClientMap.add(this);
        logger.info(session.getId() + "有新链接加入，当前链接数为：" + wsClientMap.size());
        //发送倒计时和竞标状态
        Map<String, Object> data = new HashMap<>();
        data.put("status", "kaka");
        data.put("currentRound", "/");
        data.put("timeLeft", 1.2);
        sendMessageToClient(data);
    }

    /**
     * 连接关闭
     */
    @OnClose
    public void onClose() {
        wsClientMap.remove(this);
        logger.info("有一链接关闭，当前链接数为：" + wsClientMap.size());
    }

    /**
     * 收到客户端消息
     *
     * @param message 客户端发送过来的消息
     * @param session 当前会话session
     */
    @OnMessage
    public void onMessage(String message, Session session) throws IOException, InterruptedException {
        logger.info("来终端的警情消息:" + message);
    }

    /**
     * 发生错误
     */
    @OnError
    public void onError(Session session, Throwable error) {
        logger.info("wsClientMap发生错误!");
        error.printStackTrace();
    }


    /**
     * 倒计时计算
     *
     * @param BiddingMesId 对应订单ID
     */
    public void SendCountdown(String BiddingMesId) {
        BiddingMesVO BMvo = new BiddingMesVO();
        BMvo.setBiddingMesID(BiddingMesId);
        List BMlist = biddingMesService.findBiddingMesByID(BMvo);

        if (BMlist.isEmpty()) {
            System.err.println("BMlist is empty");
            return;
        }

        BiddingMesVO voB = (BiddingMesVO) BMlist.get(0);
        Timestamp clickStartTime = voB.getClickStartTime();

        FoodRuleVO foodRuleVO = new FoodRuleVO();
        foodRuleVO.setFoodRuleID(voB.getFoodRuleID());
        List RFlist = ruleManagementService.queryRuleByID(foodRuleVO);
        FoodRuleVO voRF = (FoodRuleVO) RFlist.get(0);

        if (clickStartTime == null) {
            System.err.println("ClickStartTime is null");
            return;
        }

        // 计算每轮竞价时间和休息时间的总时长（以毫秒为单位）
        //竞价时间
        long roundsTimeMillis = voRF.getRoundsTime() * 60 * 1000;
        //休息时间
        long roundSpacingTimeMillis = voRF.getRoundSpacingTime() * 60 * 1000;
        long preparationTimeMillis = 60 * 1000; // 准备时间为1分钟

        // 计算总时长（不减去最后一次休息时间）
        long totalRoundsMillis = voRF.getTotalRounds() * (roundsTimeMillis + roundSpacingTimeMillis);

        // 计算结束时间（总时长减去最后一次休息时间，加上准备时间）
        Timestamp EndTimeDownCount = new Timestamp(clickStartTime.getTime() + totalRoundsMillis - roundSpacingTimeMillis + preparationTimeMillis);

        // 每秒更新一次倒计时
        ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);
        scheduler.scheduleAtFixedRate(() -> {
            //传递的数据
            Map<String, Object> data = new HashMap<>();
            long currentTimeMillis = System.currentTimeMillis();  // 只调用一次
            long timeLeftMillis = EndTimeDownCount.getTime() - currentTimeMillis;

            if (timeLeftMillis <= 0) {
                String status = "已结束";
                data.put("status", status);
                data.put("currentRound", "/");
                data.put("timeLeft", status);
                sendMessageToClient(data);
                scheduler.shutdown();
            } else if (timeLeftMillis > (totalRoundsMillis - roundSpacingTimeMillis)) {
                data.put("status", "准备中");
                data.put("currentRound", 0);
                String status = "即将开始";
                data.put("timeLeft", status);
                sendMessageToClient(data);
            } else {
                //一轮竞价总时长
                long roundDurationMillis = roundsTimeMillis + roundSpacingTimeMillis;
                //当前轮次剩余时间
                long currentRoundTimeLeft = timeLeftMillis % roundDurationMillis;
                // 已经过的总时间（单位：毫秒）
                long timeElapsedMillis = totalRoundsMillis - timeLeftMillis;
                //计算现在的轮数
                int completedRounds = 0;
                if (timeElapsedMillis > 0) {
                    completedRounds = (int) (timeElapsedMillis / roundDurationMillis);
                }
                // 当前轮数（从第一轮开始）
                int nowRounds = completedRounds + 1;
                data.put("currentRound", nowRounds);

                if (currentRoundTimeLeft > roundsTimeMillis) {
                    //休息中
                    data.put("status", "休息");
                } else {
                    //进行中
                    data.put("status", "进行中");
                }

                //实际发送的倒计时
                long minutes = (timeLeftMillis / 1000) / 60;
                long seconds = (timeLeftMillis / 1000) % 60;
                String timeLeft = String.format("%02d:%02d", minutes, seconds);
                data.put("timeLeft", timeLeft);
                sendMessageToClient(data);
            }
        }, 0, 1, TimeUnit.SECONDS);
    }


    private void sendMessageToClient(Object data) {
        try {
            if (session != null && session.isOpen()) {
                ObjectMapper objectMapper = new ObjectMapper();
                String jsonMessage = objectMapper.writeValueAsString(data);
                session.getBasicRemote().sendText(jsonMessage);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
